@@ -8,20 +8,9 @@
 #
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 #
-#   Copyright (c) 2019 Red Hat, Inc.
+#   Copyright Red Hat
 #
-#   This program is free software: you can redistribute it and/or
-#   modify it under the terms of the GNU General Public License as
-#   published by the Free Software Foundation, either version 2 of
-#   the License, or (at your option) any later version.
-#
-#   This program is distributed in the hope that it will be
-#   useful, but WITHOUT ANY WARRANTY; without even the implied
-#   warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
-#   PURPOSE.  See the GNU General Public License for more details.
-#
-#   You should have received a copy of the GNU General Public License
-#   along with this program. If not, see http://www.gnu.org/licenses/.
+#   SPDX-License-Identifier: GPL-2.0-or-later WITH GPL-CC-1.0
 #
 # ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 
@@ -35,6 +24,9 @@ rlJournalStart
         rlAssertRpm $PACKAGE
         rlRun "TmpDir=\$(mktemp -d)" 0 "Creating tmp directory"
         rlRun "pushd $TmpDir"
+        rlImport "tuned/basic"
+
+        tunedDisableSystemdRateLimitingStart
         rlServiceStop "tuned"
         # systemd can have some issues with quick restarts sometimes
         sleep 1
@@ -47,6 +39,7 @@ rlJournalStart
     rlPhaseEnd
 
     rlPhaseStartCleanup
+        tunedDisableSystemdRateLimitingEnd
         rlRun "popd"
         rlRun "rm -r $TmpDir" 0 "Removing tmp directory"
         rlServiceRestore "tuned"
