@@ -50,7 +50,7 @@ class Application(object):
 
 		profile_factory = profiles.Factory()
 		profile_merger = profiles.Merger()
-		profile_locator = profiles.Locator(consts.LOAD_DIRECTORIES)
+		profile_locator = profiles.Locator(self.config.get_list(consts.CFG_PROFILE_DIRS, consts.CFG_DEF_PROFILE_DIRS))
 		profile_loader = profiles.Loader(profile_locator, profile_factory, profile_merger, self.config, self.variables)
 
 		self._daemon = daemon.Daemon(unit_manager, profile_loader, profile_name, self.config, self)
@@ -71,11 +71,11 @@ class Application(object):
 		self._handle_signal(signal.SIGINT, self._controller.terminate)
 		self._handle_signal(signal.SIGTERM, self._controller.terminate)
 
-	def attach_to_dbus(self, bus_name, object_name, interface_name):
+	def attach_to_dbus(self, bus_name, object_name, interface_name, namespace):
 		if self._dbus_exporter is not None:
 			raise TunedException("DBus interface is already initialized.")
 
-		self._dbus_exporter = exports.dbus.DBusExporter(bus_name, interface_name, object_name)
+		self._dbus_exporter = exports.dbus.DBusExporter(bus_name, interface_name, object_name, namespace)
 		exports.register_exporter(self._dbus_exporter)
 
 	def attach_to_unix_socket(self):
@@ -83,7 +83,7 @@ class Application(object):
 			raise TunedException("Unix socket interface is already initialized.")
 
 		self._unix_socket_exporter = exports.unix_socket.UnixSocketExporter(self.config.get(consts.CFG_UNIX_SOCKET_PATH),
-																			self.config.get(consts.CFG_UNIX_SOCKET_SIGNAL_PATHS),
+																			self.config.get_list(consts.CFG_UNIX_SOCKET_SIGNAL_PATHS),
 																			self.config.get(consts.CFG_UNIX_SOCKET_OWNERSHIP),
 																			self.config.get_int(consts.CFG_UNIX_SOCKET_PERMISIONS),
 																			self.config.get_int(consts.CFG_UNIX_SOCKET_CONNECTIONS_BACKLOG))
@@ -211,7 +211,7 @@ class Application(object):
 		if daemon:
 			self.config.set(consts.CFG_DAEMON, True)
 		if not self.config.get_bool(consts.CFG_DAEMON, consts.CFG_DEF_DAEMON):
-			log.warn("Using one shot no daemon mode, most of the functionality will be not available, it can be changed in global config")
+			log.warning("Using one shot no daemon mode, most of the functionality will be not available, it can be changed in global config")
 		result = self._controller.run()
 		if self.config.get_bool(consts.CFG_DAEMON, consts.CFG_DEF_DAEMON):
 			exports.stop()
