@@ -5,6 +5,7 @@ from tuned.exceptions import TunedException
 import threading
 import tuned.consts as consts
 from tuned.utils.commands import commands
+from tuned.plugins import hotplug
 
 __all__ = ["Controller"]
 
@@ -71,10 +72,16 @@ class Controller(tuned.exports.interfaces.ExportableInterface):
 		self._terminate.set()
 
 	def sighup(self):
-		if not self._daemon._sighup_processing.is_set():
-			self._daemon._sighup_processing.set()
-			if not self.reload():
-				self._daemon._sighup_processing.clear()
+		if self._daemon._sighup_processing.is_set():
+			self._daemon._sighup_pending.set()
+		else:
+			do_reload = True
+			while do_reload:
+				self._daemon._sighup_processing.set()
+				self._daemon._sighup_pending.clear()
+				if not self.reload():
+					self._daemon._sighup_processing.clear()
+				do_reload = self._daemon._sighup_pending.is_set()
 
 	@exports.signal("sbs")
 	def profile_changed(self, profile_name, result, errstr):
@@ -171,7 +178,7 @@ class Controller(tuned.exports.interfaces.ExportableInterface):
 		finally:
 			if was_running:
 				if reapply:
-					log.warn("Applying previously applied (possibly out-dated) profile '%s'." % profile_name)
+					log.warning("Applying previously applied (possibly out-dated) profile '%s'." % profile_name)
 				elif not success:
 					log.info("Applying previously applied profile.")
 				self._daemon.start()
@@ -182,6 +189,8 @@ class Controller(tuned.exports.interfaces.ExportableInterface):
 	def switch_profile(self, profile_name, caller = None):
 		if caller == "":
 			return (False, "Unauthorized")
+		if not self._cmd.is_valid_name(profile_name):
+			return (False, "Invalid profile_name")
 		return self._switch_profile(profile_name, True)
 
 	@exports.export("", "(bs)")
@@ -255,8 +264,8 @@ class Controller(tuned.exports.interfaces.ExportableInterface):
 
 	@exports.export("s", "(bsss)")
 	def profile_info(self, profile_name, caller = None):
-		if caller == "":
-			return tuple(False, "", "", "")
+		if caller == "" or not self._cmd.is_valid_name(profile_name):
+			return (False, "", "", "")
 		if profile_name is None or profile_name == "":
 			profile_name = self.active_profile()
 		return tuple(self._daemon.profile_loader.profile_locator.get_profile_attrs(profile_name, [consts.PROFILE_ATTR_SUMMARY, consts.PROFILE_ATTR_DESCRIPTION], [""]))
@@ -287,7 +296,7 @@ class Controller(tuned.exports.interfaces.ExportableInterface):
 		dictionary -- {plugin_name: {parameter_name: default_value}}
 		"""
 		if caller == "":
-			return False
+			return {}
 		plugins = {}
 		for plugin_class in self._daemon.get_all_plugins():
 			plugin_name = plugin_class.__module__.split(".")[-1].split("_", 1)[1]
@@ -300,8 +309,8 @@ class Controller(tuned.exports.interfaces.ExportableInterface):
 	@exports.export("s","s")
 	def get_plugin_documentation(self, plugin_name, caller = None):
 		"""Return docstring of plugin's class"""
-		if caller == "":
-			return False
+		if caller == "" or not self._cmd.is_valid_name(plugin_name):
+			return ""
 		return self._daemon.get_plugin_documentation(str(plugin_name))
 
 	@exports.export("s","a{ss}")
@@ -314,8 +323,8 @@ class Controller(tuned.exports.interfaces.ExportableInterface):
 		Return:
 		dictionary -- {parameter_name: hint}
 		"""
-		if caller == "":
-			return False
+		if caller == "" or not self._cmd.is_valid_name(plugin_name):
+			return {}
 		return self._daemon.get_plugin_hints(str(plugin_name))
 
 	@exports.export("s", "b")
@@ -328,7 +337,7 @@ class Controller(tuned.exports.interfaces.ExportableInterface):
 		Return:
 		bool -- True on success
 		"""
-		if caller == "":
+		if caller == "" or not self._cmd.is_valid_name(path):
 			return False
 		if self._daemon._application and self._daemon._application._unix_socket_exporter:
 			self._daemon._application._unix_socket_exporter.register_signal_path(path)
@@ -342,6 +351,10 @@ class Controller(tuned.exports.interfaces.ExportableInterface):
 	def instance_acquire_devices(self, devices, instance_name, caller = None):
 		if caller == "":
 			return (False, "Unauthorized")
+		if not self._cmd.is_valid_name(devices):
+			return (False, "Invalid devices")
+		if not self._cmd.is_valid_name(instance_name):
+			return (False, "Invalid instance_name")
 		found = False
 		for instance_target in self._daemon._unit_manager.instances:
 			if instance_target.name == instance_name:
@@ -352,6 +365,10 @@ class Controller(tuned.exports.interfaces.ExportableInterface):
 			rets = "Instance '%s' not found" % instance_name
 			log.error(rets)
 			return (False, rets)
+		if not isinstance(instance_target.plugin, hotplug.Plugin):
+			rets = "Plugin '%s' does not support hotplugging or dynamic instances." % instance_target.plugin.name
+			log.error(rets)
+			return (False, rets)
 		devs = set(self._cmd.devstr2devs(devices))
 		log.debug("Instance '%s' trying to acquire devices '%s'." % (instance_target.name, str(devs)))
 		for instance in self._daemon._unit_manager.instances:
@@ -388,6 +405,8 @@ class Controller(tuned.exports.interfaces.ExportableInterface):
 		"""
 		if caller == "":
 			return (False, "Unauthorized", [])
+		if not self._cmd.is_valid_name(plugin_name):
+			return (False, "Invalid plugin_name", [])
 		if plugin_name != "" and plugin_name not in self.get_all_plugins().keys():
 			rets = "Plugin '%s' does not exist" % plugin_name
 			log.error(rets)
@@ -411,9 +430,117 @@ class Controller(tuned.exports.interfaces.ExportableInterface):
 		"""
 		if caller == "":
 			return (False, "Unauthorized", [])
+		if not self._cmd.is_valid_name(instance_name):
+			return (False, "Invalid instance_name", [])
 		for instance in self._daemon._unit_manager.instances:
 			if instance.name == instance_name:
 				return (True, "OK", sorted(list(instance.processed_devices)))
 		rets = "Instance '%s' not found" % instance_name
 		log.error(rets)
 		return (False, rets, [])
+
+	@exports.export("ssa{ss}", "(bs)")
+	def instance_create(self, plugin_name, instance_name, options, caller = None):
+		"""Dynamically create a plugin instance
+
+		Parameters:
+		plugin_name -- name of the plugin
+		instance_name -- name of the new instance
+		dict of string-string -- options for the new instance
+
+		Return:
+		bool -- True on success
+		string -- error message or "OK"
+		"""
+		if caller == "":
+			return (False, "Unauthorized")
+		if not self._cmd.is_valid_name(plugin_name):
+			return (False, "Invalid plugin_name")
+		if not self._cmd.is_valid_name(instance_name):
+			return (False, "Invalid instance_name")
+		for (key, value) in options.items():
+			if not self._cmd.is_valid_name(key) or not self._cmd.is_valid_name(value):
+				return (False, "Invalid options")
+		plugins = {p.name: p for p in self._daemon._unit_manager.plugins}
+		if not plugin_name in plugins.keys():
+			rets = "Plugin '%s' not found" % plugin_name
+			log.error(rets)
+			return (False, rets)
+		plugin = plugins[plugin_name]
+		if not isinstance(plugin, hotplug.Plugin):
+			rets = "Plugin '%s' does not support hotplugging or dynamic instances." % plugin.name
+			log.error(rets)
+			return (False, rets)
+		devices = options.pop("devices", None)
+		devices_udev_regex = options.pop("devices_udev_regex", None)
+		script_pre = options.pop("script_pre", None)
+		script_post = options.pop("script_post", None)
+		priority = int(options.pop("priority", self._daemon._unit_manager._def_instance_priority))
+		try:
+			instance = plugin.create_instance(instance_name, priority, devices, devices_udev_regex, script_pre, script_post, options)
+			plugin.initialize_instance(instance)
+			self._daemon._unit_manager.instances.append(instance)
+		except Exception as e:
+			rets = "Error creating instance '%s': %s" % (instance_name, str(e))
+			log.error(rets)
+			return (False, rets)
+		log.info("Created dynamic instance '%s' of plugin '%s'" % (instance_name, plugin_name))
+
+		plugin.assign_free_devices(instance)
+		plugin.instance_apply_tuning(instance)
+		# transfer matching devices from other instances, if the priority of the new
+		# instance is equal or higher (equal or lower priority value)
+		for other_instance in self._daemon._unit_manager.instances:
+			if (other_instance == instance or
+				other_instance.plugin != plugin or
+				instance.priority > other_instance.priority):
+				continue
+			devs_moving = plugin._get_matching_devices(instance, other_instance.processed_devices)
+			if len(devs_moving):
+				log.info("Moving devices '%s' from instance '%s' to instance '%s'." % (str(devs_moving),
+					other_instance.name, instance.name))
+				plugin._remove_devices_nocheck(other_instance, devs_moving)
+				plugin._add_devices_nocheck(instance, devs_moving)
+		return (True, "OK")
+
+	@exports.export("s", "(bs)")
+	def instance_destroy(self, instance_name, caller = None):
+		"""Destroy a dynamically created plugin instance
+
+		Parameters:
+		instance_name -- name of the new instance
+
+		Return:
+		bool -- True on success
+		string -- error message or "OK"
+		"""
+		if caller == "":
+			return (False, "Unauthorized")
+		if not self._cmd.is_valid_name(instance_name):
+			return (False, "Invalid instance_name")
+		try:
+			instance = [i for i in self._daemon._unit_manager.instances if i.name == instance_name][0]
+		except IndexError:
+			rets = "Instance '%s' not found" % instance_name
+			log.error(rets)
+			return (False, rets)
+		plugin = instance.plugin
+		if not isinstance(plugin, hotplug.Plugin):
+			rets = "Plugin '%s' does not support hotplugging or dynamic instances." % plugin.name
+			log.error(rets)
+			return (False, rets)
+		devices = instance.processed_devices.copy()
+		try:
+			plugin._remove_devices_nocheck(instance, devices)
+			self._daemon._unit_manager.instances.remove(instance)
+			plugin.instance_unapply_tuning(instance)
+			plugin.destroy_instance(instance)
+		except Exception as e:
+			rets = "Error deleting instance '%s': %s" % (instance_name, str(e))
+			log.error(rets)
+			return (False, rets)
+		log.info("Deleted instance '%s'" % instance_name)
+		for device in devices:
+			# _add_device() will find a suitable plugin instance
+			plugin._add_device(device)
+		return (True, "OK")
