@@ -13,12 +13,10 @@ cmd = commands()
 
 class AudioPlugin(hotplug.Plugin):
 	"""
-	`audio`::
-	
 	Sets audio cards power saving options. The plug-in sets the auto suspend
 	timeout for audio codecs to the value specified by the [option]`timeout`
 	option.
-	+
+
 	Currently, the `snd_hda_intel` and `snd_ac97_codec` codecs are
 	supported and the [option]`timeout` value is in seconds. To disable
 	auto suspend for these codecs, set the [option]`timeout` value
@@ -26,7 +24,7 @@ class AudioPlugin(hotplug.Plugin):
 	[option]`reset_controller` to `true`. Note that power management
 	is supported per module. Hence, the kernel module names are used as
 	device names.
-	+
+
 	.Set the timeout value to 10s and enforce the controller reset
 	====
 	----
@@ -74,7 +72,7 @@ class AudioPlugin(hotplug.Plugin):
 		return "/sys/module/%s/parameters/power_save_controller" % device
 
 	@command_set("timeout", per_device = True)
-	def _set_timeout(self, value, device, sim, remove):
+	def _set_timeout(self, value, device, instance, sim, remove):
 		try:
 			timeout = int(value)
 		except ValueError:
@@ -90,7 +88,7 @@ class AudioPlugin(hotplug.Plugin):
 			return None
 
 	@command_get("timeout")
-	def _get_timeout(self, device, ignore_missing=False):
+	def _get_timeout(self, device, instance, ignore_missing=False):
 		sys_file = self._timeout_path(device)
 		value = cmd.read_file(sys_file, no_error=ignore_missing)
 		if len(value) > 0:
@@ -98,7 +96,7 @@ class AudioPlugin(hotplug.Plugin):
 		return None
 
 	@command_set("reset_controller", per_device = True)
-	def _set_reset_controller(self, value, device, sim, remove):
+	def _set_reset_controller(self, value, device, instance, sim, remove):
 		v = cmd.get_bool(value)
 		sys_file = self._reset_controller_path(device)
 		if os.path.exists(sys_file):
@@ -109,7 +107,7 @@ class AudioPlugin(hotplug.Plugin):
 		return None
 
 	@command_get("reset_controller")
-	def _get_reset_controller(self, device, ignore_missing=False):
+	def _get_reset_controller(self, device, instance, ignore_missing=False):
 		sys_file = self._reset_controller_path(device)
 		if os.path.exists(sys_file):
 			value = cmd.read_file(sys_file)
