@@ -17,8 +17,6 @@ cpuidle_states_path = "/sys/devices/system/cpu/cpu0/cpuidle"
 
 class CPULatencyPlugin(hotplug.Plugin):
 	"""
-	`cpu`::
-	
 	Sets the CPU governor to the value specified by the [option]`governor`
 	option and dynamically changes the Power Management Quality of
 	Service (PM QoS) CPU Direct Memory Access (DMA) latency according
@@ -30,11 +28,6 @@ class CPULatencyPlugin(hotplug.Plugin):
 	character is meant to represent a logical 'or' operator. Note that the
 	same syntax is used for the [option]`energy_perf_bias` option. *TuneD*
 	will set the first governor that is available on the system.
-	+    
-	For example, with the following profile, *TuneD* will set the 'ondemand'
-	governor, if it is available. If it is not available, but the 'powersave'
-	governor is available, 'powersave' will be set. If neither of them are
-	available, the governor will not be changed.
 	+
 	.Specifying a CPU governor
 	====
@@ -42,6 +35,11 @@ class CPULatencyPlugin(hotplug.Plugin):
 	[cpu]
 	governor=ondemand|powersave
 	----
+
+	*TuneD* will set the 'ondemand'
+	governor, if it is available. If it is not available, but the 'powersave'
+	governor is available, 'powersave' will be set. If neither of them are
+	available, the governor will not be changed.
 	====
 	
 	`sampling_down_factor`:::
@@ -104,10 +102,10 @@ class CPULatencyPlugin(hotplug.Plugin):
 	`latency_low, latency_high, load_threshold`:::
 	+
 	If the CPU load is lower than the value specified by
-	the[option]`load_threshold` option, the latency is set to the value
+	the [option]`load_threshold` option, the latency is set to the value
 	specified either by the [option]`latency_high` option or by the
 	[option]`latency_low` option.
-	+
+
 	`force_latency`:::
 	You can also force the latency to a specific value and prevent it from
 	dynamically changing further. To do so, set the [option]`force_latency`
@@ -115,14 +113,16 @@ class CPULatencyPlugin(hotplug.Plugin):
 	+
 	The maximum latency value can be specified in several ways:
 	+
-	 * by a numerical value in microseconds (for example, `force_latency=10`)
-	 * as the kernel CPU idle level ID of the maximum C-state allowed
-	   (for example, force_latency = cstate.id:1)
-	 * as a case sensitive name of the maximum C-state allowed
-	   (for example, force_latency = cstate.name:C1)
-	 * by using 'None' as a fallback value to prevent errors when alternative
-	   C-state IDs/names do not exist. When 'None' is used in the alternatives
-	   pipeline, all the alternatives that follow 'None' are ignored.
+	--
+	* by a numerical value in microseconds (for example, `force_latency=10`)
+	* as the kernel CPU idle level ID of the maximum C-state allowed
+	  (for example, force_latency = cstate.id:1)
+	* as a case sensitive name of the maximum C-state allowed
+	  (for example, force_latency = cstate.name:C1)
+	* by using 'None' as a fallback value to prevent errors when alternative
+	  C-state IDs/names do not exist. When 'None' is used in the alternatives
+	  pipeline, all the alternatives that follow 'None' are ignored.
+	--
 	+
 	It is also possible to specify multiple fallback values separated by '|' as
 	the C-state names and/or IDs may not be available on some systems.
@@ -146,8 +146,8 @@ class CPULatencyPlugin(hotplug.Plugin):
 	[cpu]
 	force_latency=cstate.name:XYZ|None
 	----
-	In this case, if C-state with the name `XYZ` does not exist
-	[option]`force_latency`, no latency value will be written into the
+	In this case, if C-state with the name `XYZ` does not exist,
+	no latency value will be written into the
 	kernel's PM QoS file, and no errors will be reported due to the
 	presence of 'None'.
 	====
@@ -165,27 +165,40 @@ class CPULatencyPlugin(hotplug.Plugin):
 	Limit the minimum P-State that will be requested by the driver. It states
 	it as a percentage of the max (non-turbo) performance level.
 	====
-	+
+
 	`pm_qos_resume_latency_us`:::
 	This option allow to set specific latency for all cpus or specific ones.
+	+
+	.Configuring resume latency
 	====
 	----
 	[cpu]
 	pm_qos_resume_latency_us=n/a
 	----
 	Special value that disables C-states completely.
-	====
 	----
 	[cpu]
 	pm_qos_resume_latency_us=0
 	----
 	Allows all C-states.
-	====
 	----
 	[cpu]
 	pm_qos_resume_latency_us=100
 	----
-	Allows any C-state with a resume latency less than value.
+	Allows any C-state with a resume latency less than 100.
+	====
+
+	`boost`:::
+	The [option]`boost` option allows the CPU to boost above nominal
+	frequencies for shorts periods of time.
+	+
+	.Allowing CPU boost
+	====
+	----
+	[cpu]
+	boost=1
+	----
+	====
 	"""
 
 	def __init__(self, *args, **kwargs):
@@ -196,7 +209,8 @@ class CPULatencyPlugin(hotplug.Plugin):
 		self._is_x86 = False
 		self._is_intel = False
 		self._is_amd = False
-		self._has_energy_perf_bias = False
+		self._has_hwp_epp = False
+		self._has_energy_perf_policy_and_bias = False
 		self._has_intel_pstate = False
 		self._has_amd_pstate = False
 		self._has_pm_qos_resume_latency_us = None
@@ -236,6 +250,7 @@ class CPULatencyPlugin(hotplug.Plugin):
 			"no_turbo"             : None,
 			"pm_qos_resume_latency_us": None,
 			"energy_performance_preference" : None,
+			"boost": None,
 		}
 
 	def _check_arch(self):
@@ -259,9 +274,13 @@ class CPULatencyPlugin(hotplug.Plugin):
 		else:
 			log.info("We are running on %s (non x86)" % self._arch)
 
+		self._has_hwp_epp = consts.CFG_CPU_EPP_FLAG in self._get_cpuinfo_flags()
+
 		if self._is_intel:
-			# Check for x86_energy_perf_policy, ignore if not available / supported
-			self._check_energy_perf_bias()
+			# When hwp_epp is not supported, we check for EPB via x86_energy_perf_policy.
+			# When it is supported, EPB should be accessible via sysfs.
+			if not self._has_hwp_epp:
+				self._check_energy_perf_policy_and_bias()
 			# Check for intel_pstate
 			self._check_intel_pstate()
 
@@ -269,15 +288,15 @@ class CPULatencyPlugin(hotplug.Plugin):
 			# Check for amd-pstate
 			self._check_amd_pstate()
 
-	def _check_energy_perf_bias(self):
-		self._has_energy_perf_bias = False
+	def _check_energy_perf_policy_and_bias(self):
+		"""Check for EPB via x86_energy_perf_policy, warn if the tool is not available or EPB unsupported."""
 		retcode_unsupported = 1
 		retcode, out = self._cmd.execute(["x86_energy_perf_policy", "-r"], no_errors = [errno.ENOENT, retcode_unsupported])
 		# With recent versions of the tool, a zero exit code is
 		# returned even if EPB is not supported. The output is empty
 		# in that case, however.
 		if retcode == 0 and out != "":
-			self._has_energy_perf_bias = True
+			self._has_energy_perf_policy_and_bias = True
 		elif retcode < 0:
 			log.warning("unable to run x86_energy_perf_policy tool, ignoring CPU energy performance bias, is the tool installed?")
 		else:
@@ -317,6 +336,7 @@ class CPULatencyPlugin(hotplug.Plugin):
 	def _instance_init(self, instance):
 		instance._has_static_tuning = True
 		instance._has_dynamic_tuning = False
+		instance._load_monitor = None
 
 		# only the first instance of the plugin can control the latency
 		if list(self._instances.values())[0] == instance:
@@ -324,15 +344,12 @@ class CPULatencyPlugin(hotplug.Plugin):
 			try:
 				self._cpu_latency_fd = os.open(consts.PATH_CPU_DMA_LATENCY, os.O_WRONLY)
 			except OSError:
-				log.error("Unable to open '%s', disabling PM_QoS control" % consts.PATH_CPU_DMA_LATENCY)
+				log.info("Unable to open '%s', disabling PM_QoS control" % consts.PATH_CPU_DMA_LATENCY)
 				self._has_pm_qos = False
 			self._latency = None
 
 			if instance.options["force_latency"] is None and instance.options["pm_qos_resume_latency_us"] is None:
-				instance._load_monitor = self._monitors_repository.create("load", None)
 				instance._has_dynamic_tuning = True
-			else:
-				instance._load_monitor = None
 
 			self._check_arch()
 		else:
@@ -351,12 +368,17 @@ class CPULatencyPlugin(hotplug.Plugin):
 			if instance._load_monitor is not None:
 				self._monitors_repository.delete(instance._load_monitor)
 
+	def _instance_init_dynamic(self, instance):
+		super(CPULatencyPlugin, self)._instance_init_dynamic(instance)
+		if instance._first_instance:
+			instance._load_monitor = self._monitors_repository.create("load", None)
+
 	def _get_intel_pstate_attr(self, attr):
 		return self._cmd.read_file("/sys/devices/system/cpu/intel_pstate/%s" % attr, None).strip()
 
 	def _set_intel_pstate_attr(self, attr, val):
 		if val is not None:
-			self._cmd.write_to_file("/sys/devices/system/cpu/intel_pstate/%s" % attr, val)
+			self._cmd.write_to_file("/sys/devices/system/cpu/intel_pstate/%s" % attr, val, ignore_same=True)
 
 	def _getset_intel_pstate_attr(self, attr, value):
 		if value is None:
@@ -384,8 +406,8 @@ class CPULatencyPlugin(hotplug.Plugin):
 				instance.options["max_perf_pct"])
 			self._max_perf_pct_save = self._getset_intel_pstate_attr(
 				"max_perf_pct", new_value)
-			new_value = self._variables.expand(
-				instance.options["no_turbo"])
+			new_value = self._cmd.get_bool(self._variables.expand(
+				instance.options["no_turbo"]))
 			self._no_turbo_save = self._getset_intel_pstate_attr(
 				"no_turbo", new_value)
 
@@ -504,7 +526,7 @@ class CPULatencyPlugin(hotplug.Plugin):
 		return self._cmd.read_file("/sys/devices/system/cpu/%s/cpufreq/scaling_available_governors" % device).strip().split()
 
 	@command_set("governor", per_device=True)
-	def _set_governor(self, governors, device, sim, remove):
+	def _set_governor(self, governors, device, instance, sim, remove):
 		if not self._check_cpu_can_change_governor(device):
 			return None
 		governors = str(governors)
@@ -521,19 +543,19 @@ class CPULatencyPlugin(hotplug.Plugin):
 					log.info("setting governor '%s' on cpu '%s'"
 							% (governor, device))
 					self._cmd.write_to_file("/sys/devices/system/cpu/%s/cpufreq/scaling_governor"
-							% device, governor, no_error = [errno.ENOENT] if remove else False)
+							% device, governor, no_error = [errno.ENOENT] if remove else False, ignore_same=True)
 				break
 			elif not sim:
 				log.debug("Ignoring governor '%s' on cpu '%s', it is not supported"
 						% (governor, device))
 		else:
-			log.warn("None of the scaling governors is supported: %s"
+			log.warning("None of the scaling governors is supported: %s"
 					% ", ".join(governors))
 			governor = None
 		return governor
 
 	@command_get("governor")
-	def _get_governor(self, device, ignore_missing=False):
+	def _get_governor(self, device, instance, ignore_missing=False):
 		governor = None
 		if not self._check_cpu_can_change_governor(device):
 			return None
@@ -550,7 +572,7 @@ class CPULatencyPlugin(hotplug.Plugin):
 		return "/sys/devices/system/cpu/cpufreq/%s/sampling_down_factor" % governor
 
 	@command_set("sampling_down_factor", per_device = True, priority = 10)
-	def _set_sampling_down_factor(self, sampling_down_factor, device, sim, remove):
+	def _set_sampling_down_factor(self, sampling_down_factor, device, instance, sim, remove):
 		val = None
 
 		# hack to clear governors map when the profile starts unloading
@@ -560,7 +582,7 @@ class CPULatencyPlugin(hotplug.Plugin):
 			self._governors_map.clear()
 
 		self._governors_map[device] = None
-		governor = self._get_governor(device)
+		governor = self._get_governor(device, instance)
 		if governor is None:
 			log.debug("ignoring sampling_down_factor setting for CPU '%s', cannot match governor" % device)
 			return None
@@ -577,8 +599,8 @@ class CPULatencyPlugin(hotplug.Plugin):
 		return val
 
 	@command_get("sampling_down_factor")
-	def _get_sampling_down_factor(self, device, ignore_missing=False):
-		governor = self._get_governor(device, ignore_missing=ignore_missing)
+	def _get_sampling_down_factor(self, device, instance, ignore_missing=False):
+		governor = self._get_governor(device, instance, ignore_missing=ignore_missing)
 		if governor is None:
 			return None
 		path = self._sampling_down_factor_path(governor)
@@ -595,6 +617,9 @@ class CPULatencyPlugin(hotplug.Plugin):
 				return_err = True)
 		return (retcode, err_msg)
 
+	def _pstate_boost_path(self, cpu_id):
+		return "/sys/devices/system/cpu/cpufreq/policy%s/boost" % cpu_id
+
 	def _pstate_preference_path(self, cpu_id, available = False):
 		return "/sys/devices/system/cpu/cpufreq/policy%s/energy_performance_%s" % (cpu_id, "available_preferences" if available else "preference")
 
@@ -602,7 +627,7 @@ class CPULatencyPlugin(hotplug.Plugin):
 		return "/sys/devices/system/cpu/cpu%s/power/energy_perf_bias" % cpu_id
 
 	@command_set("energy_perf_bias", per_device=True)
-	def _set_energy_perf_bias(self, energy_perf_bias, device, sim, remove):
+	def _set_energy_perf_bias(self, energy_perf_bias, device, instance, sim, remove):
 		if not self._is_cpu_online(device):
 			log.debug("%s is not online, skipping" % device)
 			return None
@@ -611,14 +636,14 @@ class CPULatencyPlugin(hotplug.Plugin):
 
 		# It should be writen straight to sysfs energy_perf_bias file if requested on newer processors
 		# see rhbz#2095829
-		if consts.CFG_CPU_EPP_FLAG in self._get_cpuinfo_flags():
+		if self._has_hwp_epp:
 			energy_perf_bias_path = self._energy_perf_bias_path(cpu_id)
 			if os.path.exists(energy_perf_bias_path):
 				if not sim:
 					for val in vals:
 						val = val.strip()
 						if self._cmd.write_to_file(energy_perf_bias_path, val, \
-							no_error = [errno.ENOENT] if remove else False):
+							no_error = [errno.ENOENT] if remove else False, ignore_same=True):
 								log.info("energy_perf_bias successfully set to '%s' on cpu '%s'"
 										 % (val, device))
 								break
@@ -631,7 +656,7 @@ class CPULatencyPlugin(hotplug.Plugin):
 				log.error("Failed to set energy_perf_bias on cpu '%s' because energy_perf_bias file does not exist."
 						  % device)
 				return None
-		elif self._has_energy_perf_bias:
+		elif self._has_energy_perf_policy_and_bias:
 			if not sim:
 				for val in vals:
 					val = val.strip()
@@ -681,17 +706,17 @@ class CPULatencyPlugin(hotplug.Plugin):
 				}.get(self._try_parse_num(s), s)
 
 	@command_get("energy_perf_bias")
-	def _get_energy_perf_bias(self, device, ignore_missing=False):
+	def _get_energy_perf_bias(self, device, instance, ignore_missing=False):
 		energy_perf_bias = None
 		if not self._is_cpu_online(device):
 			log.debug("%s is not online, skipping" % device)
 			return None
 		cpu_id = device.lstrip("cpu")
-		if consts.CFG_CPU_EPP_FLAG in self._get_cpuinfo_flags():
+		if self._has_hwp_epp:
 			energy_perf_bias_path = self._energy_perf_bias_path(cpu_id)
 			if os.path.exists(energy_perf_bias_path):
 				energy_perf_bias = self._energy_perf_policy_to_human_v2(self._cmd.read_file(energy_perf_bias_path))
-		elif self._has_energy_perf_bias:
+		elif self._has_energy_perf_policy_and_bias:
 			retcode, lines = self._cmd.execute(["x86_energy_perf_policy", "-c", cpu_id, "-r"])
 			if retcode == 0:
 				for line in lines.splitlines():
@@ -716,7 +741,7 @@ class CPULatencyPlugin(hotplug.Plugin):
 		return self._has_pm_qos_resume_latency_us
 
 	@command_set("pm_qos_resume_latency_us", per_device=True)
-	def _set_pm_qos_resume_latency_us(self, pm_qos_resume_latency_us, device, sim, remove):
+	def _set_pm_qos_resume_latency_us(self, pm_qos_resume_latency_us, device, instance, sim, remove):
 		if not self._is_cpu_online(device):
 			log.debug("%s is not online, skipping" % device)
 			return None
@@ -732,7 +757,7 @@ class CPULatencyPlugin(hotplug.Plugin):
 		return latency
 
 	@command_get("pm_qos_resume_latency_us")
-	def _get_pm_qos_resume_latency_us(self, device, ignore_missing=False):
+	def _get_pm_qos_resume_latency_us(self, device, instance, ignore_missing=False):
 		if not self._is_cpu_online(device):
 			log.debug("%s is not online, skipping" % device)
 			return None
@@ -740,8 +765,39 @@ class CPULatencyPlugin(hotplug.Plugin):
 			return None
 		return self._cmd.read_file(self._pm_qos_resume_latency_us_path(device), no_error=ignore_missing).strip()
 
+	@command_set("boost", per_device=True)
+	def _set_boost(self, boost, device, instance, sim, remove):
+		if not self._is_cpu_online(device):
+			log.debug("%s is not online, skipping" % device)
+			return None
+		cpu_id = device.lstrip("cpu")
+		if os.path.exists(self._pstate_boost_path(cpu_id)):
+			if not sim:
+				if boost == "0" or boost == "1":
+					self._cmd.write_to_file(self._pstate_boost_path(cpu_id), boost, \
+						no_error = [errno.ENOENT] if remove else False, ignore_same=True)
+					log.info("Setting boost value '%s' for cpu '%s'" % (boost, device))
+				else:
+					log.error("Failed to set boost on cpu '%s'. Is the value in the profile correct?" % device)
+			return str(boost)
+		else:
+			log.debug("boost file missing, which can happen on pre 6.11 kernels.")
+		return None
+
+	@command_get("boost")
+	def _get_boost(self, device, instance, ignore_missing=False):
+		if not self._is_cpu_online(device):
+			log.debug("%s is not online, skipping" % device)
+			return None
+		cpu_id = device.lstrip("cpu")
+		if os.path.exists(self._pstate_boost_path(cpu_id)):
+			return self._cmd.read_file(self._pstate_boost_path(cpu_id)).strip()
+		else:
+			log.debug("boost file missing, which can happen on pre 6.11 kernels.")
+		return None
+
 	@command_set("energy_performance_preference", per_device=True)
-	def _set_energy_performance_preference(self, energy_performance_preference, device, sim, remove):
+	def _set_energy_performance_preference(self, energy_performance_preference, device, instance, sim, remove):
 		if not self._is_cpu_online(device):
 			log.debug("%s is not online, skipping" % device)
 			return None
@@ -751,13 +807,19 @@ class CPULatencyPlugin(hotplug.Plugin):
 			if not sim:
 				avail_vals = set(self._cmd.read_file(self._pstate_preference_path(cpu_id, True)).split())
 				for val in vals:
-					if val in avail_vals:
+					try:
+						val = int(val)
+						valid = 0 <= val < 256
+						val = str(val)
+					except ValueError:
+						valid = val in avail_vals
+					if valid:
 						self._cmd.write_to_file(self._pstate_preference_path(cpu_id), val, \
-							no_error = [errno.ENOENT] if remove else False)
+							no_error = [errno.ENOENT] if remove else False, ignore_same=True)
 						log.info("Setting energy_performance_preference value '%s' for cpu '%s'" % (val, device))
 						break
 					else:
-						log.warn("energy_performance_preference value '%s' unavailable for cpu '%s'" % (val, device))
+						log.warning("energy_performance_preference value '%s' unavailable for cpu '%s'" % (val, device))
 				else:
 					log.error("Failed to set energy_performance_preference on cpu '%s'. Is the value in the profile correct?"
 							  % device)
@@ -767,7 +829,7 @@ class CPULatencyPlugin(hotplug.Plugin):
 		return None
 
 	@command_get("energy_performance_preference")
-	def _get_energy_performance_preference(self, device, ignore_missing=False):
+	def _get_energy_performance_preference(self, device, instance, ignore_missing=False):
 		if not self._is_cpu_online(device):
 			log.debug("%s is not online, skipping" % device)
 			return None
