@@ -11,29 +11,26 @@ log = tuned.logs.get()
 
 class DiskPlugin(hotplug.Plugin):
 	"""
-	`disk`::
-	
 	Plug-in for tuning various block device options. This plug-in can also
 	dynamically change the advanced power management and spindown timeout
 	setting for a drive according to the current drive utilization. The
 	dynamic tuning is controlled by the [option]`dynamic` and the global
 	[option]`dynamic_tuning` option in `tuned-main.conf`.
-	+
+
 	The disk plug-in operates on all supported block devices unless a
 	comma separated list of [option]`devices` is passed to it.
-	+
-	.Operate only on the sda block device
+
+	.Operate only on the `sda` block device
 	====
 	----
 	[disk]
-	# Comma separated list of devices, all devices if commented out.
 	devices=sda
 	----
 	====
-	+
+
 	The [option]`elevator` option sets the Linux I/O scheduler.
-	+
-	.Use the bfq I/O scheduler on xvda block device
+
+	.Use the bfq I/O scheduler on the `xvda` block device
 	====
 	----
 	[disk]
@@ -41,21 +38,21 @@ class DiskPlugin(hotplug.Plugin):
 	elevator=bfq
 	----
 	====
-	+
+
 	The [option]`scheduler_quantum` option only applies to the CFQ I/O
 	scheduler. It defines the number of I/O requests that CFQ sends to
 	one device at one time, essentially limiting queue depth. The default
 	value is 8 requests. The device being used may support greater queue
 	depth, but increasing the value of quantum will also increase latency,
 	especially for large sequential write work loads.
-	+
+
 	The [option]`apm` option sets the Advanced Power Management feature
 	on drives that support it. It corresponds to using the `-B` option of
 	the `hdparm` utility. The [option]`spindown` option puts the drive
 	into idle (low-power) mode, and also sets the standby (spindown)
 	timeout for the drive. It corresponds to using `-S` option of the
 	`hdparm` utility.
-	+
+
 	.Use a medium-agressive power management with spindown
 	====
 	----
@@ -64,7 +61,7 @@ class DiskPlugin(hotplug.Plugin):
 	spindown=6
 	----
 	====
-	+
+
 	The [option]`readahead` option controls how much extra data the
 	operating system reads from disk when performing sequential
 	I/O operations. Increasing the `readahead` value might improve
@@ -73,7 +70,7 @@ class DiskPlugin(hotplug.Plugin):
 	can be adjusted to sectors by specifying the suffix 's'. If the
 	suffix is specified, there must be at least one space between the
 	number and suffix (for example, `readahead=8192 s`).
-	+
+
 	.Set the `readahead` to 4MB unless already set to a higher value
 	====
 	----
@@ -100,32 +97,36 @@ class DiskPlugin(hotplug.Plugin):
 		self._devices_supported = True
 		self._use_hdparm = True
 		self._free_devices = set()
-		self._hdparm_apm_devices = set()
+		self._hdparm_apm_device_support = dict()
 		for device in self._hardware_inventory.get_devices("block"):
 			if self._device_is_supported(device):
 				self._free_devices.add(device.sys_name)
-				if self._use_hdparm and self._is_hdparm_apm_supported(device.sys_name):
-					self._hdparm_apm_devices.add(device.sys_name)
-
 		self._assigned_devices = set()
 
 	def _get_device_objects(self, devices):
 		return [self._hardware_inventory.get_device("block", x) for x in devices]
 
 	def _is_hdparm_apm_supported(self, device):
+		if not self._use_hdparm:
+			return False
+		if device in self._hdparm_apm_device_support:
+			return self._hdparm_apm_device_support[device]
 		(rc, out, err_msg) = self._cmd.execute(["hdparm", "-C", "/dev/%s" % device], \
 				no_errors = [errno.ENOENT], return_err=True)
 		if rc == -errno.ENOENT:
-			log.warn("hdparm command not found, ignoring for other devices")
+			log.warning("hdparm command not found, ignoring for other devices")
 			self._use_hdparm = False
 			return False
 		elif rc:
 			log.info("Device '%s' not supported by hdparm" % device)
 			log.debug("(rc: %s, msg: '%s')" % (rc, err_msg))
+			self._hdparm_apm_device_support[device] = False
 			return False
 		elif "unknown" in out:
 			log.info("Driver for device '%s' does not support apm command" % device)
+			self._hdparm_apm_device_support[device] = False
 			return False
+		self._hdparm_apm_device_support[device] = True
 		return True
 
 	@classmethod
@@ -180,24 +181,23 @@ class DiskPlugin(hotplug.Plugin):
 		self._apm_errcnt = 0
 		self._spindown_errcnt = 0
 
-		if self._option_bool(instance.options["dynamic"]):
-			instance._has_dynamic_tuning = True
-			instance._load_monitor = \
-					self._monitors_repository.create(
-					"disk", instance.assigned_devices)
-			instance._device_idle = {}
-			instance._stats = {}
-			instance._idle = {}
-			instance._spindown_change_delayed = {}
-		else:
-			instance._has_dynamic_tuning = False
-			instance._load_monitor = None
+		instance._load_monitor = None
+		instance._has_dynamic_tuning = self._option_bool(instance.options["dynamic"])
 
 	def _instance_cleanup(self, instance):
 		if instance._load_monitor is not None:
 			self._monitors_repository.delete(instance._load_monitor)
 			instance._load_monitor = None
 
+	def _instance_init_dynamic(self, instance):
+		super(DiskPlugin, self)._instance_init_dynamic(instance)
+		instance._device_idle = {}
+		instance._stats = {}
+		instance._idle = {}
+		instance._spindown_change_delayed = {}
+		instance._load_monitor = self._monitors_repository.create(
+						"disk", instance.assigned_devices)
+
 	def _update_errcnt(self, rc, spindown):
 		if spindown:
 			s = "spindown"
@@ -211,7 +211,7 @@ class DiskPlugin(hotplug.Plugin):
 			cnt = 0
 		elif rc == -errno.ENOENT:
 			self._spindown_errcnt = self._apm_errcnt = consts.ERROR_THRESHOLD + 1
-			log.warn("hdparm command not found, ignoring future set_apm / set_spindown commands")
+			log.warning("hdparm command not found, ignoring future set_apm / set_spindown commands")
 			return
 		else:
 			cnt += 1
@@ -233,7 +233,7 @@ class DiskPlugin(hotplug.Plugin):
 		return not "standby" in out and not "sleeping" in out
 
 	def _instance_update_dynamic(self, instance, device):
-		if not device in self._hdparm_apm_devices:
+		if not self._is_hdparm_apm_supported(device):
 			return
 		load = instance._load_monitor.get_device_load(device)
 		if load is None:
@@ -316,7 +316,7 @@ class DiskPlugin(hotplug.Plugin):
 		# At the moment we support dynamic tuning just for devices compatible with hdparm apm commands
 		# If in future will be added new functionality not connected to this command,
 		# it is needed to change it here
-		if device not in self._hdparm_apm_devices:
+		if not self._is_hdparm_apm_supported(device):
 			log.info("There is no dynamic tuning available for device '%s' at time" % device)
 		else:
 			super(DiskPlugin, self)._instance_apply_dynamic(instance, device)
@@ -335,7 +335,7 @@ class DiskPlugin(hotplug.Plugin):
 		return self._sysfs_path(device, "queue/scheduler")
 
 	@command_set("elevator", per_device=True)
-	def _set_elevator(self, value, device, sim, remove):
+	def _set_elevator(self, value, device, instance, sim, remove):
 		sys_file = self._elevator_file(device)
 		if not sim:
 			self._cmd.write_to_file(sys_file, value, \
@@ -343,15 +343,15 @@ class DiskPlugin(hotplug.Plugin):
 		return value
 
 	@command_get("elevator")
-	def _get_elevator(self, device, ignore_missing=False):
+	def _get_elevator(self, device, instance, ignore_missing=False):
 		sys_file = self._elevator_file(device)
 		# example of scheduler file content:
 		# noop deadline [cfq]
 		return self._cmd.get_active_option(self._cmd.read_file(sys_file, no_error=ignore_missing))
 
 	@command_set("apm", per_device=True)
-	def _set_apm(self, value, device, sim, remove):
-		if device not in self._hdparm_apm_devices:
+	def _set_apm(self, value, device, instance, sim, remove):
+		if not self._is_hdparm_apm_supported(device):
 			if not sim:
 				log.info("apm option is not supported for device '%s'" % device)
 				return None
@@ -366,8 +366,8 @@ class DiskPlugin(hotplug.Plugin):
 			return None
 
 	@command_get("apm")
-	def _get_apm(self, device, ignore_missing=False):
-		if device not in self._hdparm_apm_devices:
+	def _get_apm(self, device, instance, ignore_missing=False):
+		if not self._is_hdparm_apm_supported(device):
 			if not ignore_missing:
 				log.info("apm option is not supported for device '%s'" % device)
 			return None
@@ -390,8 +390,8 @@ class DiskPlugin(hotplug.Plugin):
 		return value
 
 	@command_set("spindown", per_device=True)
-	def _set_spindown(self, value, device, sim, remove):
-		if device not in self._hdparm_apm_devices:
+	def _set_spindown(self, value, device, instance, sim, remove):
+		if not self._is_hdparm_apm_supported(device):
 			if not sim:
 				log.info("spindown option is not supported for device '%s'" % device)
 				return None
@@ -406,8 +406,8 @@ class DiskPlugin(hotplug.Plugin):
 			return None
 
 	@command_get("spindown")
-	def _get_spindown(self, device, ignore_missing=False):
-		if device not in self._hdparm_apm_devices:
+	def _get_spindown(self, device, instance, ignore_missing=False):
+		if not self._is_hdparm_apm_supported(device):
 			if not ignore_missing:
 				log.info("spindown option is not supported for device '%s'" % device)
 			return None
@@ -429,7 +429,7 @@ class DiskPlugin(hotplug.Plugin):
 		return v
 
 	@command_set("readahead", per_device=True)
-	def _set_readahead(self, value, device, sim, remove):
+	def _set_readahead(self, value, device, instance, sim, remove):
 		sys_file = self._readahead_file(device)
 		val = self._parse_ra(value)
 		if val is None:
@@ -441,7 +441,7 @@ class DiskPlugin(hotplug.Plugin):
 		return val
 
 	@command_get("readahead")
-	def _get_readahead(self, device, ignore_missing=False):
+	def _get_readahead(self, device, instance, ignore_missing=False):
 		sys_file = self._readahead_file(device)
 		value = self._cmd.read_file(sys_file, no_error=ignore_missing).strip()
 		if len(value) == 0:
@@ -449,31 +449,31 @@ class DiskPlugin(hotplug.Plugin):
 		return int(value)
 
 	@command_custom("readahead_multiply", per_device=True)
-	def _multiply_readahead(self, enabling, multiplier, device, verify, ignore_missing):
+	def _multiply_readahead(self, enabling, multiplier, device, verify, ignore_missing, instance):
 		if verify:
 			return None
 		storage_key = self._storage_key(
 				command_name = "readahead_multiply",
 				device_name = device)
 		if enabling:
-			old_readahead = self._get_readahead(device)
+			old_readahead = self._get_readahead(device, instance)
 			if old_readahead is None:
 				return
 			new_readahead = int(float(multiplier) * old_readahead)
 			self._storage.set(storage_key, old_readahead)
-			self._set_readahead(new_readahead, device, False)
+			self._set_readahead(new_readahead, device, instance, False, False)
 		else:
 			old_readahead = self._storage.get(storage_key)
 			if old_readahead is None:
 				return
-			self._set_readahead(old_readahead, device, False)
+			self._set_readahead(old_readahead, device, instance, False, False)
 			self._storage.unset(storage_key)
 
 	def _scheduler_quantum_file(self, device):
 		return self._sysfs_path(device, "queue/iosched/quantum")
 
 	@command_set("scheduler_quantum", per_device=True)
-	def _set_scheduler_quantum(self, value, device, sim, remove):
+	def _set_scheduler_quantum(self, value, device, instance, sim, remove):
 		sys_file = self._scheduler_quantum_file(device)
 		if not sim:
 			self._cmd.write_to_file(sys_file, "%d" % int(value), \
@@ -481,7 +481,7 @@ class DiskPlugin(hotplug.Plugin):
 		return value
 
 	@command_get("scheduler_quantum")
-	def _get_scheduler_quantum(self, device, ignore_missing=False):
+	def _get_scheduler_quantum(self, device, instance, ignore_missing=False):
 		sys_file = self._scheduler_quantum_file(device)
 		value = self._cmd.read_file(sys_file, no_error=ignore_missing).strip()
 		if len(value) == 0:
