@@ -8,7 +8,12 @@ import tuned.logs
 import re
 from subprocess import *
 import threading
-import perf
+# perf is optional
+try:
+	import perf
+except ImportError:
+# if perf is unavailable, it will be disabled later
+	pass
 import select
 import tuned.consts as consts
 import procfs
@@ -136,11 +141,9 @@ class SchedulerUtilsSchedutils(SchedulerUtils):
 
 class SchedulerPlugin(base.Plugin):
 	r"""
-	`scheduler`::
-	
 	Allows tuning of scheduling priorities, process/thread/IRQ
 	affinities, and CPU isolation.
-	+
+
 	To prevent processes/threads/IRQs from using certain CPUs, use
 	the [option]`isolated_cores` option. It changes process/thread
 	affinities, IRQs affinities and it sets `default_smp_affinity`
@@ -154,11 +157,11 @@ class SchedulerPlugin(base.Plugin):
 	cmd`) are matched against that expression. Profile rollback allows
 	all matching processes and threads to run on all CPUs and restores
 	the IRQ settings prior to the profile application.
-	+
+
 	Multiple regular expressions for [option]`ps_whitelist`
 	and [option]`ps_blacklist` options are allowed and separated by
 	`;`. Quoted semicolon `\;` is taken literally.
-	+
+
 	.Isolate CPUs 2-4
 	====
 	----
@@ -169,23 +172,24 @@ class SchedulerPlugin(base.Plugin):
 	Isolate CPUs 2-4 while ignoring processes and threads matching
 	`ps_blacklist` regular expressions.
 	====
+
 	The [option]`irq_process` option controls whether the scheduler plugin
 	applies the `isolated_cores` parameter to IRQ affinities. The default
 	value is `true`, which means that the scheduler plugin will move all
 	possible IRQs away from the isolated cores. When `irq_process` is set
 	to `false`, the plugin will not change any IRQ affinities.
-	====
+
 	The [option]`default_irq_smp_affinity` option controls the values
 	*TuneD* writes to `/proc/irq/default_smp_affinity`. The file specifies
 	default affinity mask that applies to all non-active IRQs. Once an
 	IRQ is allocated/activated its affinity bitmask will be set to the
 	default mask.
-	+
+
 	The following values are supported:
+
+	* `calc`
 	+
-	--
-	`calc`::
-	Content of `/proc/irq/default_smp_affinity` will be calculated
+	The content of `/proc/irq/default_smp_affinity` will be calculated
 	from the `isolated_cores` parameter. Non-isolated cores
 	are calculated as an inversion of the `isolated_cores`. Then
 	the intersection of the non-isolated cores and the previous
@@ -194,13 +198,16 @@ class SchedulerPlugin(base.Plugin):
 	an empty set, then just the non-isolated cores are written to
 	`/proc/irq/default_smp_affinity`. This behavior is the default if
 	the parameter `default_irq_smp_affinity` is omitted.
-	`ignore`::
-	*TuneD* will not touch `/proc/irq/default_smp_affinity`.
-	explicit cpulist::
-	The cpulist (such as 1,3-4) is unpacked and written directly to
-	`/proc/irq/default_smp_affinity`.
-	--
+
+	* `ignore`
 	+
+	*TuneD* will not touch `/proc/irq/default_smp_affinity`.
+
+	* an explicit cpulist
+	+
+	The cpulist (such as `1,3-4`) is unpacked and written directly to
+	`/proc/irq/default_smp_affinity`.
+
 	.An explicit CPU list to set the default IRQ smp affinity to CPUs 0 and 2
 	====
 	----
@@ -209,48 +216,49 @@ class SchedulerPlugin(base.Plugin):
 	default_irq_smp_affinity=0,2
 	----
 	====
+
 	To adjust scheduling policy, priority and affinity for a group of
 	processes/threads, use the following syntax.
-	+
-	[subs="+quotes,+macros"]
+
+	[subs="quotes"]
 	----
 	group.__groupname__=__rule_prio__:__sched__:__prio__:__affinity__:__regex__
 	----
-	+
-	where `__rule_prio__` defines internal *TuneD* priority of the
+
+	Here, `__rule_prio__` defines internal *TuneD* priority of the
 	rule. Rules are sorted based on priority. This is needed for
 	inheritence to be able to reorder previously defined rules. Equal
 	`__rule_prio__` rules should be processed in the order they were
 	defined. However, this is Python interpreter dependant. To disable
 	an inherited rule for `__groupname__` use:
-	+
-	[subs="+quotes,+macros"]
+
+	[subs="quotes"]
 	----
 	group.__groupname__=
 	----
-	+
+
 	`__sched__` must be one of:
 	*`f`* for FIFO,
 	*`b`* for batch,
 	*`r`* for round robin,
 	*`o`* for other,
 	*`*`* do not change.
-	+
+
 	`__affinity__` is CPU affinity in hexadecimal. Use `*` for no change.
-	+
+
 	`__prio__` scheduling priority (see `chrt -m`).
-	+
-	`__regex__` is Python regular expression. It is matched against the output of
-	+
-	[subs="+quotes,+macros"]
+
+	`__regex__` is Python regular expression. It is matched against the output of:
+
+	[subs="quotes"]
 	----
 	ps -eo cmd
 	----
-	+
+
 	Any given process name may match more than one group. In such a case,
 	the priority and scheduling policy are taken from the last matching
 	`__regex__`.
-	+
+
 	.Setting scheduling policy and priorities to kernel threads and watchdog
 	====
 	----
@@ -259,7 +267,7 @@ class SchedulerPlugin(base.Plugin):
 	group.watchdog=0:f:99:*:\[watchdog.*\]
 	----
 	====
-	+
+
 	The scheduler plug-in uses perf event loop to catch newly created
 	processes. By default it listens to `perf.RECORD_COMM` and
 	`perf.RECORD_EXIT` events. By setting [option]`perf_process_fork`
@@ -271,14 +279,14 @@ class SchedulerPlugin(base.Plugin):
 	pose a significant CPU overhead, the [option]`perf_process_fork`
 	option parameter is set to `false` by default. Due to this, child
 	processes are not processed by the scheduler plug-in.
-	+
+
 	The CPU overhead of the scheduler plugin can be mitigated by using
 	the scheduler [option]`runtime` option and setting it to `0`. This
 	will completely disable the dynamic scheduler functionality and the
 	perf events will not be monitored and acted upon. The disadvantage
 	ot this approach is the procees/thread tuning will be done only at
 	profile application.
-	+
+
 	.Disabling the scheduler dynamic functionality
 	====
 	----
@@ -287,7 +295,7 @@ class SchedulerPlugin(base.Plugin):
 	isolated_cores=1,3
 	----
 	====
-	+
+
 	NOTE: For perf events, memory mapped buffer is used. Under heavy load
 	the buffer may overflow. In such cases the `scheduler` plug-in
 	may start missing events and failing to process some newly created
@@ -297,24 +305,24 @@ class SchedulerPlugin(base.Plugin):
 	of 2, the nearest higher power of 2 value is calculated from it
 	and this calculated value used. If the [option]`perf_mmap_pages`
 	option is omitted, the default kernel value is used.
-	+
+
 	The scheduler plug-in supports process/thread confinement using
 	cgroups v1.
-	+
+
 	[option]`cgroup_mount_point` option specifies the path to mount the
 	cgroup filesystem or where *TuneD* expects it to be mounted. If unset,
 	`/sys/fs/cgroup/cpuset` is expected.
-	+
+
 	If [option]`cgroup_groups_init` option is set to `1` *TuneD*
 	will create (and remove) all cgroups defined with the `cgroup*`
 	options. This is the default behavior. If it is set to `0` the
 	cgroups need to be preset by other means.
-	+
+
 	If [option]`cgroup_mount_point_init` option is set to `1`,
 	*TuneD* will create (and remove) the cgroup mountpoint. It implies
 	`cgroup_groups_init = 1`. If set to `0` the cgroups mount point
 	needs to be preset by other means. This is the default behavior.
-	+
+
 	The [option]`cgroup_for_isolated_cores` option is the cgroup
 	name used for the [option]`isolated_cores` option functionality. For
 	example, if a system has 4 CPUs, `isolated_cores=1` means that all
@@ -324,24 +332,24 @@ class SchedulerPlugin(base.Plugin):
 	the specified cgroup and move all the matching processes/threads to
 	this group. If this option is unset, classic cpuset affinity using
 	`sched_setaffinity()` will be used.
-	+
+
 	[option]`cgroup.__cgroup_name__` option defines affinities for
 	arbitrary cgroups. Even hierarchic cgroups can be used, but the
 	hieararchy needs to be specified in the correct order. Also *TuneD*
 	does not do any sanity checks here, with the exception that it forces
 	the cgroup to be under [option]`cgroup_mount_point`.
-	+
+
 	The syntax of the scheduler option starting with `group.` has been
 	augmented to use `cgroup.__cgroup_name__` instead of the hexadecimal
 	`__affinity__`. The matching processes will be moved to the cgroup
 	`__cgroup_name__`. It is also possible to use cgroups which have
 	not been defined by the [option]`cgroup.` option as described above,
 	i.e. cgroups not managed by *TuneD*.
-	+
+
 	All cgroup names are sanitized by replacing all all dots (`.`) with
 	slashes (`/`). This is to prevent the plug-in from writing outside
 	[option]`cgroup_mount_point`.
-	+
+
 	.Using cgroups v1 with the scheduler plug-in
 	====
 	----
@@ -367,6 +375,7 @@ class SchedulerPlugin(base.Plugin):
 	CPU affinity 0,2-3 to the `cpuset.cpus` control file of the `group`
 	and move all the matching processes/threads to this cgroup.
 	====
+
 	Option [option]`cgroup_ps_blacklist` allows excluding processes
 	which belong to the blacklisted cgroups. The regular expression specified
 	by this option is matched against cgroup hierarchies from
@@ -374,10 +383,10 @@ class SchedulerPlugin(base.Plugin):
 	are separated by commas ',' prior to regular expression matching. The
 	following is an example of content against which the regular expression
 	is matched against: `10:hugetlb:/,9:perf_event:/,8:blkio:/`
-	+
+
 	Multiple regular expressions can be separated by semicolon ';'. The
 	semicolon represents a logical 'or' operator.
-	+
+
 	.Cgroup-based exclusion of processes from the scheduler
 	====
 	----
@@ -385,21 +394,19 @@ class SchedulerPlugin(base.Plugin):
 	isolated_cores=1
 	cgroup_ps_blacklist=:/daemons\b
 	----
-	
 	The scheduler plug-in will move all processes away from core 1 except processes which
 	belong to cgroup '/daemons'. The '\b' is a regular expression
 	metacharacter that matches a word boundary.
-	
 	----
 	[scheduler]
 	isolated_cores=1
 	cgroup_ps_blacklist=\b8:blkio:
 	----
-	
 	The scheduler plug-in will exclude all processes which belong to a cgroup
 	with hierarchy-ID 8 and controller-list blkio.
 	====
-	Recent kernels moved some `sched_` and `numa_balancing_` kernel run-time
+
+	Kernels 5.13 and newer moved some `sched_` and `numa_balancing_` kernel run-time
 	parameters from `/proc/sys/kernel`, managed by the `sysctl` utility, to
 	`debugfs`, typically mounted under `/sys/kernel/debug`.  TuneD provides an
 	abstraction mechanism for the following parameters via the scheduler plug-in:
@@ -410,9 +417,13 @@ class SchedulerPlugin(base.Plugin):
 	[option]`numa_balancing_scan_period_min_ms`,
 	[option]`numa_balancing_scan_period_max_ms` and
 	[option]`numa_balancing_scan_size_mb`.
-	Based on the kernel used, TuneD will write the specified value to the correct
-	location.
-	+
+	Moreover in kernel 6.6 and newer support for the `sched_wakeup_granularity_ns` and
+	`sched_latency_ns` were removed. The `sched_min_granularity_ns` was renamed to
+	`sched_base_slice_ns`. Based on the kernel used, TuneD will write the specified
+	value to the correct location or ignore it. For the compatibility the alias
+	[option]`sched_base_slice_ns` was added, but the [option]`sched_min_granularity_ns`
+	can be still used instead.
+
 	.Set tasks' "cache hot" value for migration decisions.
 	====
 	----
@@ -430,6 +441,12 @@ class SchedulerPlugin(base.Plugin):
 	====
 	"""
 
+	_dict_sched_knob_map = {
+		"wakeup_granularity_ns": "",
+		"min_granularity_ns": "base_slice_ns",
+		"latency_ns": "",
+	}
+
 	def __init__(self, monitor_repository, storage_factory, hardware_inventory, device_matcher, device_matcher_udev, plugin_instance_factory, global_cfg, variables):
 		super(SchedulerPlugin, self).__init__(monitor_repository, storage_factory, hardware_inventory, device_matcher, device_matcher_udev, plugin_instance_factory, global_cfg, variables)
 		self._has_dynamic_options = True
@@ -446,8 +463,17 @@ class SchedulerPlugin(base.Plugin):
 		# default is to whitelist all and blacklist none
 		self._ps_whitelist = ".*"
 		self._ps_blacklist = ""
+		self._kthread_process = True
 		self._cgroup_ps_blacklist_re = ""
-		self._cpus = perf.cpu_map()
+		# perf is optional, if unavailable, it will be disabled later
+		try:
+			self._cpus = perf.cpu_map()
+		except (NameError, AttributeError):
+			cpus = self._cmd.read_file(consts.SYSFS_CPUS_PRESENT_PATH)
+			# it's different type than perf.cpu_map(), but without perf we use it as iterable
+			# which should be compatible, fallback to single core CPU if sysfs is unavailable
+			self._cpus = self._cmd.cpulist_unpack(cpus) if cpus else [ 0 ]
+
 		self._scheduler_storage_key = self._storage_key(
 				command_name = "scheduler")
 		self._irq_process = True
@@ -532,6 +558,8 @@ class SchedulerPlugin(base.Plugin):
 					instance._evlist.mmap(pages = perf_mmap_pages)
 			# no perf
 			except:
+				log.warning("python-perf unavailable, disabling perf support and " \
+					"runtime tuning, you can try to (re)install python(3)-perf package")
 				instance._runtime_tuning = False
 
 	def _instance_cleanup(self, instance):
@@ -550,11 +578,13 @@ class SchedulerPlugin(base.Plugin):
 			"cgroup_ps_blacklist": None,
 			"ps_whitelist": None,
 			"ps_blacklist": None,
+			"kthread_process": True,
 			"irq_process": True,
 			"default_irq_smp_affinity": "calc",
 			"perf_mmap_pages": None,
 			"perf_process_fork": "false",
 			"sched_min_granularity_ns": None,
+			"sched_base_slice_ns": None,
 			"sched_latency_ns": None,
 			"sched_wakeup_granularity_ns": None,
 			"sched_tunable_scaling": None,
@@ -586,6 +616,8 @@ class SchedulerPlugin(base.Plugin):
 		processes = {}
 		for proc in ps.values():
 			try:
+				if not self._kthread_process and self._is_kthread(proc):
+					continue
 				cmd = self._get_cmdline(proc)
 				pid = proc["pid"]
 				processes[pid] = cmd
@@ -644,15 +676,24 @@ class SchedulerPlugin(base.Plugin):
 	def _is_kthread(self, process):
 		return process["stat"]["flags"] & procfs.pidstat.PF_KTHREAD != 0
 
+	def _process_in_blacklisted_cgroup(self, process):
+		if self._cgroup_ps_blacklist_re == "":
+			return False
+		return re.search(self._cgroup_ps_blacklist_re, self._get_stat_cgroup(process)) is not None
+
 	# Returns True if we can ignore a failed affinity change of
 	# a process with the given PID and therefore not report it as an error.
-	def _ignore_set_affinity_error(self, pid):
+	def _ignore_set_affinity_error(self, process):
+		pid = process.pid
 		try:
-			process = procfs.process(pid)
 			if process["stat"]["state"] == "Z":
 				log.debug("Affinity of zombie task with PID %d could not be changed."
 						% pid)
 				return True
+			if self._process_in_blacklisted_cgroup(process):
+				log.debug("Affinity of task with PID %d could not be changed, the task was moved into a blacklisted cgroup."
+						% pid)
+				return True
 			if process["stat"].is_bound_to_cpu():
 				if self._is_kthread(process):
 					log.debug("Affinity of kernel thread with PID %d cannot be changed, the task's affinity mask is fixed."
@@ -661,6 +702,9 @@ class SchedulerPlugin(base.Plugin):
 					log.warning("Affinity of task with PID %d cannot be changed, the task's affinity mask is fixed."
 							% pid)
 				return True
+			log.info("Task %d cmdline: %s" % (pid, self._get_cmdline(process)))
+			log.info("Task %d cgroup: %s" % (pid, self._get_stat_cgroup(process)))
+			log.info("Task %d affinity: %s" % (pid, list(self._scheduler_utils.get_affinity(pid))))
 		except (OSError, IOError) as e:
 			if e.errno == errno.ENOENT or e.errno == errno.ESRCH:
 				log.debug("Failed to get task info for PID %d, the task vanished."
@@ -1053,6 +1097,9 @@ class SchedulerPlugin(base.Plugin):
 
 	def _add_pid(self, instance, pid, r):
 		try:
+			proc = procfs.process(pid)
+			if not self._kthread_process and self._is_kthread(proc):
+				return
 			cmd = self._get_cmdline(pid)
 		except (OSError, IOError) as e:
 			if e.errno == errno.ENOENT \
@@ -1098,14 +1145,17 @@ class SchedulerPlugin(base.Plugin):
 						event = instance._evlist.read_on_cpu(cpu)
 						if event:
 							read_events = True
-							if event.type == perf.RECORD_COMM or \
-								(self._perf_process_fork_value and event.type == perf.RECORD_FORK):
+							if isinstance(event, perf.comm_event) or (
+								self._perf_process_fork_value
+								and isinstance(event, perf.task_event)
+								and event.type == perf.RECORD_FORK
+							):
 								self._add_pid(instance, int(event.tid), r)
-							elif event.type == perf.RECORD_EXIT:
+							elif isinstance(event, perf.task_event) and event.type == perf.RECORD_EXIT:
 								self._remove_pid(instance, int(event.tid))
 
 	@command_custom("cgroup_ps_blacklist", per_device = False)
-	def _cgroup_ps_blacklist(self, enabling, value, verify, ignore_missing):
+	def _cgroup_ps_blacklist(self, enabling, value, verify, ignore_missing, instance):
 		# currently unsupported
 		if verify:
 			return None
@@ -1113,7 +1163,7 @@ class SchedulerPlugin(base.Plugin):
 			self._cgroup_ps_blacklist_re = "|".join(["(%s)" % v for v in re.split(r"(?<!\\);", str(value))])
 
 	@command_custom("ps_whitelist", per_device = False)
-	def _ps_whitelist(self, enabling, value, verify, ignore_missing):
+	def _ps_whitelist(self, enabling, value, verify, ignore_missing, instance):
 		# currently unsupported
 		if verify:
 			return None
@@ -1121,15 +1171,23 @@ class SchedulerPlugin(base.Plugin):
 			self._ps_whitelist = "|".join(["(%s)" % v for v in re.split(r"(?<!\\);", str(value))])
 
 	@command_custom("ps_blacklist", per_device = False)
-	def _ps_blacklist(self, enabling, value, verify, ignore_missing):
+	def _ps_blacklist(self, enabling, value, verify, ignore_missing, instance):
 		# currently unsupported
 		if verify:
 			return None
 		if enabling and value is not None:
 			self._ps_blacklist = "|".join(["(%s)" % v for v in re.split(r"(?<!\\);", str(value))])
 
+	@command_custom("kthread_process", per_device = False)
+	def _kthread_process(self, enabling, value, verify, ignore_missing, instance):
+		# currently unsupported
+		if verify:
+			return None
+		if enabling and value is not None:
+			self._kthread_process = self._cmd.get_bool(value) == "1"
+
 	@command_custom("irq_process", per_device = False)
-	def _irq_process(self, enabling, value, verify, ignore_missing):
+	def _irq_process(self, enabling, value, verify, ignore_missing, instance):
 		# currently unsupported
 		if verify:
 			return None
@@ -1137,7 +1195,7 @@ class SchedulerPlugin(base.Plugin):
 			self._irq_process = self._cmd.get_bool(value) == "1"
 
 	@command_custom("default_irq_smp_affinity", per_device = False)
-	def _default_irq_smp_affinity(self, enabling, value, verify, ignore_missing):
+	def _default_irq_smp_affinity(self, enabling, value, verify, ignore_missing, instance):
 		# currently unsupported
 		if verify:
 			return None
@@ -1148,7 +1206,7 @@ class SchedulerPlugin(base.Plugin):
 				self._default_irq_smp_affinity_value = self._cmd.cpulist_unpack(value)
 
 	@command_custom("perf_process_fork", per_device = False)
-	def _perf_process_fork(self, enabling, value, verify, ignore_missing):
+	def _perf_process_fork(self, enabling, value, verify, ignore_missing, instance):
 		# currently unsupported
 		if verify:
 			return None
@@ -1165,13 +1223,17 @@ class SchedulerPlugin(base.Plugin):
 		return res
 
 	def _set_affinity(self, pid, affinity):
+		process = procfs.process(pid)
+		if self._process_in_blacklisted_cgroup(process):
+			log.debug("Not setting CPU affinity of PID %d, the task belongs to a blacklisted cgroup." % pid)
+			return
 		log.debug("Setting CPU affinity of PID %d to '%s'." % (pid, affinity))
 		try:
 			self._scheduler_utils.set_affinity(pid, affinity)
 		# Workaround for old python-schedutils (pre-0.4) which
 		# incorrectly raised SystemError instead of OSError
 		except (SystemError, OSError) as e:
-			if not self._ignore_set_affinity_error(pid):
+			if not self._ignore_set_affinity_error(process):
 				log.error("Failed to set affinity of PID %d to '%s': %s"
 						% (pid, affinity, e))
 
@@ -1183,14 +1245,14 @@ class SchedulerPlugin(base.Plugin):
 		return affinity3
 
 	def _set_all_obj_affinity(self, objs, affinity, threads = False):
-		psl = [v for v in objs if re.search(self._ps_whitelist,
+		psl = objs
+		if not self._kthread_process:
+			psl = [v for v in psl if not self._is_kthread(v)]
+		psl = [v for v in psl if re.search(self._ps_whitelist,
 				self._get_stat_comm(v)) is not None]
 		if self._ps_blacklist != "":
 			psl = [v for v in psl if re.search(self._ps_blacklist,
 					self._get_stat_comm(v)) is None]
-		if self._cgroup_ps_blacklist_re != "":
-			psl = [v for v in psl if re.search(self._cgroup_ps_blacklist_re,
-					self._get_stat_cgroup(v)) is None]
 		psd = dict([(v.pid, v) for v in psl])
 		for pid in psd:
 			try:
@@ -1360,7 +1422,7 @@ class SchedulerPlugin(base.Plugin):
 		return res
 
 	@command_custom("isolated_cores", per_device = False, priority = 10)
-	def _isolated_cores(self, enabling, value, verify, ignore_missing):
+	def _isolated_cores(self, enabling, value, verify, ignore_missing, instance):
 		affinity = None
 		self._affinity = None
 		if value is not None:
@@ -1394,118 +1456,152 @@ class SchedulerPlugin(base.Plugin):
 			# _instance_unapply_static()
 			if self._irq_process:
 				self._restore_all_irq_affinity()
+		return True
+
+	def _sched_assembly_path(self, prefix, namespace, knob):
+		if prefix == "":
+			path = "%s/%s" % (namespace, knob)
+		else:
+			path = "%s/%s/%s" % (prefix, namespace, knob)
+		return "/sys/kernel/debug/%s" % path
+
+	# map to kernel 6.6 paths, "" means that knob was dropped
+	def _sched_assembly_path2(self, path, prefix, namespace, knob):
+		lpath = path
+		if namespace == "sched":
+			lknob = self._dict_sched_knob_map.get(knob)
+			if lknob is not None:
+				if lknob:
+					lpath = self._sched_assembly_path(prefix, namespace, lknob)
+				else:
+					lpath = ""
+		return lpath
 
 	def _get_sched_knob_path(self, prefix, namespace, knob):
 		key = "%s_%s_%s" % (prefix, namespace, knob)
 		path = self._sched_knob_paths_cache.get(key)
-		if path:
+		if path or path == "":
 			return path
 		path = "/proc/sys/kernel/%s_%s" % (namespace, knob)
 		if not os.path.exists(path):
-			if prefix == "":
-				path = "%s/%s" % (namespace, knob)
-			else:
-				path = "%s/%s/%s" % (prefix, namespace, knob)
-			path = "/sys/kernel/debug/%s" % path
-			if self._secure_boot_hint is None:
+			path = self._sched_assembly_path(prefix, namespace, knob)
+			# kernel 6.6 drops and renames some knobs
+			if not os.path.exists(path):
+				path = self._sched_assembly_path2(path, prefix, namespace, knob)
+			if path != "" and self._secure_boot_hint is None:
 				self._secure_boot_hint = True
 		self._sched_knob_paths_cache[key] = path
 		return path
 
 	def _get_sched_knob(self, prefix, namespace, knob):
-		data = self._cmd.read_file(self._get_sched_knob_path(prefix, namespace, knob), err_ret = None)
-		if data is None:
-			log.error("Error reading '%s'" % knob)
-			if self._secure_boot_hint:
-				log.error("This may not work with Secure Boot or kernel_lockdown (this hint is logged only once)")
-				self._secure_boot_hint = False
+		data = None
+		path = self._get_sched_knob_path(prefix, namespace, knob)
+		if path != "":
+			data = self._cmd.read_file(path, err_ret = None)
+			if data is None:
+				log.error("Error reading '%s'" % knob)
+				if self._secure_boot_hint:
+					log.error("This may not work with Secure Boot or kernel_lockdown (this hint is logged only once)")
+					self._secure_boot_hint = False
 		return data
 
 	def _set_sched_knob(self, prefix, namespace, knob, value, sim, remove = False):
 		if value is None:
 			return None
+		path = self._get_sched_knob_path(prefix, namespace, knob)
+		if not path:
+			log.debug("knob '%s' ignored, unsupported by kernel" % knob)
+			return None
 		if not sim:
-			if not self._cmd.write_to_file(self._get_sched_knob_path(prefix, namespace, knob), value, \
+			if not self._cmd.write_to_file(path, value, \
 				no_error = [errno.ENOENT] if remove else False):
 					log.error("Error writing value '%s' to '%s'" % (value, knob))
 		return value
 
 	@command_get("sched_min_granularity_ns")
-	def _get_sched_min_granularity_ns(self):
+	def _get_sched_min_granularity_ns(self, instance):
 		return self._get_sched_knob("", "sched", "min_granularity_ns")
 
 	@command_set("sched_min_granularity_ns")
-	def _set_sched_min_granularity_ns(self, value, sim, remove):
+	def _set_sched_min_granularity_ns(self, value, instance, sim, remove):
 		return self._set_sched_knob("", "sched", "min_granularity_ns", value, sim, remove)
 
+	@command_get("sched_base_slice_ns")
+	def _get_sched_base_slice_ns(self, instance):
+		return self._get_sched_min_granularity_ns(instance)
+
+	@command_set("sched_base_slice_ns")
+	def _set_sched_base_slice_ns(self, value, instance, sim, remove):
+		return self._set_sched_min_granularity_ns(value, instance, sim, remove)
+
 	@command_get("sched_latency_ns")
-	def _get_sched_latency_ns(self):
+	def _get_sched_latency_ns(self, instance):
 		return self._get_sched_knob("", "sched", "latency_ns")
 
 	@command_set("sched_latency_ns")
-	def _set_sched_latency_ns(self, value, sim, remove):
+	def _set_sched_latency_ns(self, value, instance, sim, remove):
 		return self._set_sched_knob("", "sched", "latency_ns", value, sim, remove)
 
 	@command_get("sched_wakeup_granularity_ns")
-	def _get_sched_wakeup_granularity_ns(self):
+	def _get_sched_wakeup_granularity_ns(self, instance):
 		return self._get_sched_knob("", "sched", "wakeup_granularity_ns")
 
 	@command_set("sched_wakeup_granularity_ns")
-	def _set_sched_wakeup_granularity_ns(self, value, sim, remove):
+	def _set_sched_wakeup_granularity_ns(self, value, instance, sim, remove):
 		return self._set_sched_knob("", "sched", "wakeup_granularity_ns", value, sim, remove)
 
 	@command_get("sched_tunable_scaling")
-	def _get_sched_tunable_scaling(self):
+	def _get_sched_tunable_scaling(self, instance):
 		return self._get_sched_knob("", "sched", "tunable_scaling")
 
 	@command_set("sched_tunable_scaling")
-	def _set_sched_tunable_scaling(self, value, sim, remove):
+	def _set_sched_tunable_scaling(self, value, instance, sim, remove):
 		return self._set_sched_knob("", "sched", "tunable_scaling", value, sim, remove)
 
 	@command_get("sched_migration_cost_ns")
-	def _get_sched_migration_cost_ns(self):
+	def _get_sched_migration_cost_ns(self, instance):
 		return self._get_sched_knob("", "sched", "migration_cost_ns")
 
 	@command_set("sched_migration_cost_ns")
-	def _set_sched_migration_cost_ns(self, value, sim, remove):
+	def _set_sched_migration_cost_ns(self, value, instance, sim, remove):
 		return self._set_sched_knob("", "sched", "migration_cost_ns", value, sim, remove)
 
 	@command_get("sched_nr_migrate")
-	def _get_sched_nr_migrate(self):
+	def _get_sched_nr_migrate(self, instance):
 		return self._get_sched_knob("", "sched", "nr_migrate")
 
 	@command_set("sched_nr_migrate")
-	def _set_sched_nr_migrate(self, value, sim, remove):
+	def _set_sched_nr_migrate(self, value, instance, sim, remove):
 		return self._set_sched_knob("", "sched", "nr_migrate", value, sim, remove)
 
 	@command_get("numa_balancing_scan_delay_ms")
-	def _get_numa_balancing_scan_delay_ms(self):
+	def _get_numa_balancing_scan_delay_ms(self, instance):
 		return self._get_sched_knob("sched", "numa_balancing", "scan_delay_ms")
 
 	@command_set("numa_balancing_scan_delay_ms")
-	def _set_numa_balancing_scan_delay_ms(self, value, sim, remove):
+	def _set_numa_balancing_scan_delay_ms(self, value, instance, sim, remove):
 		return self._set_sched_knob("sched", "numa_balancing", "scan_delay_ms", value, sim, remove)
 
 	@command_get("numa_balancing_scan_period_min_ms")
-	def _get_numa_balancing_scan_period_min_ms(self):
+	def _get_numa_balancing_scan_period_min_ms(self, instance):
 		return self._get_sched_knob("sched", "numa_balancing", "scan_period_min_ms")
 
 	@command_set("numa_balancing_scan_period_min_ms")
-	def _set_numa_balancing_scan_period_min_ms(self, value, sim, remove):
+	def _set_numa_balancing_scan_period_min_ms(self, value, instance, sim, remove):
 		return self._set_sched_knob("sched", "numa_balancing", "scan_period_min_ms", value, sim, remove)
 
 	@command_get("numa_balancing_scan_period_max_ms")
-	def _get_numa_balancing_scan_period_max_ms(self):
+	def _get_numa_balancing_scan_period_max_ms(self, instance):
 		return self._get_sched_knob("sched", "numa_balancing", "scan_period_max_ms")
 
 	@command_set("numa_balancing_scan_period_max_ms")
-	def _set_numa_balancing_scan_period_max_ms(self, value, sim, remove):
+	def _set_numa_balancing_scan_period_max_ms(self, value, instance, sim, remove):
 		return self._set_sched_knob("sched", "numa_balancing", "scan_period_max_ms", value, sim, remove)
 
 	@command_get("numa_balancing_scan_size_mb")
-	def _get_numa_balancing_scan_size_mb(self):
+	def _get_numa_balancing_scan_size_mb(self, instance):
 		return self._get_sched_knob("sched", "numa_balancing", "scan_size_mb")
 
 	@command_set("numa_balancing_scan_size_mb")
-	def _set_numa_balancing_scan_size_mb(self, value, sim, remove):
+	def _set_numa_balancing_scan_size_mb(self, value, instance, sim, remove):
 		return self._set_sched_knob("sched", "numa_balancing", "scan_size_mb", value, sim, remove)
