@@ -11,17 +11,15 @@ log = tuned.logs.get()
 
 class SCSIHostPlugin(hotplug.Plugin):
 	"""
-	`scsi_host`::
-	
 	Tunes options for SCSI hosts.
-	+
+
 	The plug-in sets Aggressive Link Power Management (ALPM) to the value specified
 	by the [option]`alpm` option. The option takes one of three values:
 	`min_power`, `medium_power` and `max_performance`.
-	+
+
 	NOTE: ALPM is only available on SATA controllers that use the Advanced
 	Host Controller Interface (AHCI).
-	+
+
 	.ALPM setting when extended periods of idle time are expected
 	====
 	----
@@ -86,7 +84,7 @@ class SCSIHostPlugin(hotplug.Plugin):
 		return os.path.join("/sys/class/scsi_host/", str(device), "link_power_management_policy")
 
 	@command_set("alpm", per_device = True)
-	def _set_alpm(self, policy, device, sim, remove):
+	def _set_alpm(self, policy, device, instance, sim, remove):
 		if policy is None:
 			return None
 		policy_file = self._get_alpm_policy_file(device)
@@ -100,7 +98,7 @@ class SCSIHostPlugin(hotplug.Plugin):
 		return policy
 
 	@command_get("alpm")
-	def _get_alpm(self, device, ignore_missing=False):
+	def _get_alpm(self, device, instance, ignore_missing=False):
 		policy_file = self._get_alpm_policy_file(device)
 		policy = self._cmd.read_file(policy_file, no_error = True).strip()
 		return policy if policy != "" else None
